/*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
// (C) PicoPeta Simputers Pvt. Ltd. (2003)
// Author : Saugata Das
// Extras

#include "pwrap.h"
#include "pdefs.h"

/// [Alchemy Command]
void pw_alchemy_command(px_widget *toplevel, int command, const char *data, ...)
{
	if(data) {
		char *txt=NULL;
		va_list ap;

		va_start(ap, data);
		txt = _pw_my_printf(data, ap);
		va_end(ap);

		ac_control_win(command, txt, xdisplay, toplevel->win);

		if(txt)
			free(txt);
	} else {
		ac_control_win(command, NULL, xdisplay, toplevel->win);
	}
}

/// [Message Box]
messagebox_button pw_messagebox(	px_widget *toplevel,
							messagebox_button type, messagebox_icon icon,
							const char *title, const char *mesg
						)
{
	messagebox *mb;
	messagebox_button ret;
	
	mb =  messagebox_create(toplevel);

	mb->set_message(mb, mesg);
	mb->set_type(mb, type);
	mb->set_icon(mb, icon);
	mb->set_title(mb, title);
	mb->message_widget->specific.label.set_font(mb->message_widget, PW_BOLD_FONT);
	mb->message_widget->redraw(mb->message_widget);

	ret = mb->get_answer(mb);
	
	mb->destroy(mb);
	
	return ret;
}

/// [Screen Dimension & Orientation]
unsigned pw_get_display_width(pw_widget window)
{
#ifdef __arm__
	XRRScreenConfiguration *sc;
	Rotation current_rotation;
	int dwidth;

	sc = XRRGetScreenInfo(xdisplay, window->win);
	XRRConfigRotations(sc, &current_rotation);
	
	if(current_rotation&0x01 || current_rotation&0x04)
		dwidth = 240;
	else
		dwidth = 320;

	XRRFreeScreenConfigInfo(sc);

	return dwidth;
#else
	return DisplayWidth(xdisplay, screen);
#endif
}

unsigned pw_get_display_height(pw_widget window)
{
#ifdef __arm__
	XRRScreenConfiguration *sc;
	Rotation current_rotation;
	int dheight;

	sc = XRRGetScreenInfo(xdisplay, window->win);
	XRRConfigRotations(sc, &current_rotation);
	
	if(current_rotation&0x01 || current_rotation&0x04)
		dheight = 320;
	else
		dheight = 240;

	XRRFreeScreenConfigInfo(sc);

	return dheight;
#else
	return DisplayHeight(xdisplay, screen);
#endif
}

void pw_set_display_orientation(pw_widget window, unsigned orientation)
{
#ifdef __arm__
	switch(orientation) {
		case PW_ORIENT_TOP:
			system("xrandr -o top");
			break;
		case PW_ORIENT_BOTTOM:
			system("xrandr -o bottom");
			break;
		case PW_ORIENT_LEFT:
			system("xrandr -o left");
			break;
		case PW_ORIENT_RIGHT:
			system("xrandr -o right");
			break;
	}
#endif
}

unsigned pw_get_display_orientation(pw_widget window)
{
#ifdef __arm__
	XRRScreenConfiguration *sc;
	Rotation current_rotation;
	int orient;

	sc = XRRGetScreenInfo(xdisplay, window->win);
	XRRConfigRotations(sc, &current_rotation);

	if(current_rotation&0x01)
		orient = PW_ORIENT_TOP;
	if(current_rotation&0x02)
		orient = PW_ORIENT_RIGHT;
	if(current_rotation&0x04)
		orient = PW_ORIENT_BOTTOM;
	if(current_rotation&0x08)
		orient = PW_ORIENT_LEFT;

	XRRFreeScreenConfigInfo(sc);

	return orient;
#else
	return PW_ORIENT_TOP;
#endif
}

void pw_toggle_fullscreen(pw_widget window)
{
	ewmh_command_win("_NET_WM_STATE_FULLSCREEN", _NET_WM_STATE_TOGGLE, xdisplay, window->win);
}

