/*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
// (C) PicoPeta Simputers Pvt. Ltd. (2003)
// Author : Saugata Das
// Generic functions

#include "pwrap.h"
#include "pdefs.h"

void pw_show(pw_widget widget)
{
	widget->show(widget);
}

void pw_show_many(pw_widget widget, ...)
{
	va_list ap;
	pw_widget w;

	if(widget) {
		widget->show(widget);
		va_start(ap, widget);
		while(1) {
			w = va_arg(ap, pw_widget);
			if(w)
				w->show(w);
			else
				break;
		}
		va_end(ap);
	}
}

void pw_hide(pw_widget widget)
{
	widget->hide(widget);
}

void pw_hide_many(pw_widget widget, ...)
{
	va_list ap;
	pw_widget w;

	if(widget) {
		widget->hide(widget);
		va_start(ap, widget);
		while(1) {
			w = va_arg(ap, pw_widget);
			if(w)
				w->hide(w);
			else
				break;
		}
		va_end(ap);
	}
}

void pw_raise(pw_widget widget)
{
	widget->raise(widget);
}

void pw_lower(pw_widget widget)
{
	widget->lower(widget);
}

void pw_set_text(pw_widget widget, const char *text, ...)
{
	char *txt=NULL;
	va_list ap;
	
	va_start(ap, text);
	txt = _pw_my_printf(text, ap);
	va_end(ap);

	switch(widget->wtype) {
		case BUTTON:
			widget->specific.button.set_text(widget, txt);
			break;
		case CHECKBUTTON:
			widget->specific.checkbutton.set_text(widget, txt);
			break;
		case RADIOBUTTON:
			widget->specific.radiobutton.set_text(widget, txt);
			break;
		case LABEL:
			widget->specific.label.set_text(widget, txt);
			break;
		case ENTRY:
			widget->specific.entry.set_text(widget, txt);
			break;
		case TEXT:
			widget->specific.text.set_text(widget, txt);
			break;
		default:
			break;
	}

	if(txt)
		free(txt);

	_pw_redraw_queue_add(widget, 1);
}

void pw_set_bgimage_file(pw_widget widget, const char *file, ...)
{
	char *txt=NULL;
	va_list ap;
	
	va_start(ap, file);
	txt = _pw_my_printf(file, ap);
	va_end(ap);

	widget->set_background_pixmap_from_file(widget, txt);
	_pw_redraw_queue_add(widget, 1);

	if(txt)
		free(txt);
}

void pw_set_fgcolor(pw_widget widget, const char *color)
{
	px_color col;

	col.type = SOLID;
	widget_xcolor_from_str(&col.first_color, color);

	switch(widget->wtype) {
		case LABEL:
			widget->specific.label.set_normal_color(widget, color);
			break;
		case BUTTON:
			widget->specific.button.set_normal_font_color(widget, color);
			break;
		case RADIOBUTTON:
			widget->specific.radiobutton.set_normal_font_color(widget, color);
			break;
		case CHECKBUTTON:
			widget->specific.checkbutton.set_normal_font_color(widget, color);
			break;
		case ENTRY:
			widget->specific.entry.set_foreground_color(widget, color);
			break;
		case PROGRESSBAR:
			widget->specific.progressbar.set_font_color(widget, color);
			break;
		case TEXT:
			widget->specific.text.set_foreground_color(widget, color);
			break;
		case TXTLB:
			widget->specific.txtlb.change_lb_color(widget, TXTLB_FOREGROUND, (char*)color);
			break;
		case SKETCH:
			pw_sketch_pencolor(widget, color);
			break;
		case COMBOBOX:
			widget->specific.combobox.change_color(widget, COMBO_LABEL_FOREGROUND, color);
			break;
		default:
			break;
	}
	
	_pw_redraw_queue_add(widget, 1);
}

void pw_set_bgcolor(pw_widget widget, const char *color)
{
	px_color col;

	col.type = SOLID;
	widget_xcolor_from_str(&col.first_color, color);

	if(widget->wtype == SKETCH) {
		unsigned long scol = 0;
		char str[20];

		sprintf(str, "0x%02x%02x%02x", 
				col.first_color.red*255/0xffff, 
				col.first_color.green*255/0xffff, 
				col.first_color.blue*255/0xffff);
		sscanf(str, "%lx", &scol);

		widget->specific.sketch.set_bg_color(widget, scol);
	} else if(widget->wtype == TXTLB) {
		widget->specific.txtlb.change_lb_color(widget,TXTLB_ROW_BACKGROUND,color);
	} else if(widget->wtype == COMBOBOX) {
		widget->specific.combobox.change_color(widget, COMBO_LABEL_BACKGROUND, color);
	} else {
		widget->set_background_color(widget, &col);
	}
	
	_pw_redraw_queue_add(widget, 0);
}

void pw_set_bgtransparent(pw_widget widget)
{
	px_color col;

	col.type = TRANSPARENT;

	widget->set_background_color(widget, &col);
	_pw_redraw_queue_add(widget, 1);
}

void pw_set_position(pw_widget widget, int x, int y)
{
	widget->set_position(widget, x, y);
	_pw_redraw_queue_add(widget, 1);
}

void pw_get_position(pw_widget widget, int *x, int *y)
{
	widget->get_position(widget, x, y);
}

void pw_set_size(pw_widget widget, unsigned long width, unsigned long height)
{
	widget->set_size(widget, width, height);
	_pw_redraw_queue_add(widget, 1);
}

void pw_get_size(pw_widget widget, unsigned long *width, unsigned long *height)
{
	widget->get_size(widget, width, height);
}

const char *pw_get_text(pw_widget widget)
{
	switch(widget->wtype) {
		case BUTTON:
			return widget->specific.button.get_text(widget);
		case CHECKBUTTON:
			return widget->specific.checkbutton.get_text(widget);
		case RADIOBUTTON:
			return widget->specific.radiobutton.get_text(widget);
		case LABEL:
			return widget->specific.label.get_text(widget);
		case ENTRY:
			return widget->specific.entry.text;
		case TEXT:
			return widget->specific.text.get_text(widget);
		default:
			return NULL;
	}
}

const char *pw_get_tag(pw_widget widget)
{
	return widget->tag;
}

unsigned pw_get_bordertype(pw_widget widget)
{
	switch(widget->border) {
		case NO_BORDER:
			return PW_NO_BORDER;
		case RECTANGLE_BORDER:
			return PW_RECTANGLE_BORDER;
		case BEVELLED_BORDER:
			return PW_BEVELLED_BORDER;
		case ELLIPTIC_BORDER:
			return PW_ELLIPTIC_BORDER;
		default:
			return PW_NO_BORDER;
	}
}

void pw_enable(pw_widget widget)
{
	widget->set_state(widget, NORMAL);
	_pw_redraw_queue_add(widget, 1);
}

void pw_enable_many(pw_widget widget, ...)
{
	va_list ap;
	pw_widget w;

	if(widget) {
		widget->set_state(widget, NORMAL);
		va_start(ap, widget);
		while(1) {
			w = va_arg(ap, pw_widget);
			if(w)
				w->set_state(w, NORMAL);
			else
				break;
		}
		va_end(ap);

		widget->redraw(widget);
		va_start(ap, widget);
		while(1) {
			w = va_arg(ap, pw_widget);
			if(w)
				w->redraw(w);
			else
				break;
		}
		va_end(ap);
	}
}

void pw_disable(pw_widget widget)
{
	widget->set_state(widget, DISABLED);
	_pw_redraw_queue_add(widget, 1);
}

void pw_disable_many(pw_widget widget, ...)
{
	va_list ap;
	pw_widget w;

	if(widget) {
		widget->set_state(widget, DISABLED);
		va_start(ap, widget);
		while(1) {
			w = va_arg(ap, pw_widget);
			if(w)
				w->set_state(w, DISABLED);
			else
				break;
		}
		va_end(ap);

		widget->redraw(widget);
		va_start(ap, widget);
		while(1) {
			w = va_arg(ap, pw_widget);
			if(w)
				w->redraw(w);
			else
				break;
		}
		va_end(ap);
	}
}

void pw_set_title(pw_widget widget, const char *title)
{
	if(widget->wtype==TOPLEVEL)
		widget->specific.toplevel.set_title(widget, title);
}

void pw_set_wrap(pw_widget widget, unsigned wraptype)
{
	unsigned wrtype = 99999;
	
	switch(wraptype) {
		case PW_NO_WRAP:
			wrtype = NO_WRAP;
			break;
		case PW_CHAR_WRAP:
			wrtype = WRAP_CHAR;
			break;
		case PW_WORD_WRAP:
			wrtype = WRAP_WORD;
			break;
	}
	
	switch(widget->wtype) {
		case BUTTON:
			widget->specific.button.set_wrap_type(widget, wrtype);
			break;
		case CHECKBUTTON:
			widget->specific.checkbutton.set_wrap_type(widget, wrtype);
			break;
		case RADIOBUTTON:
			widget->specific.radiobutton.set_wrap_type(widget, wrtype);
			break;
		case LABEL:
			widget->specific.label.set_wrap_type(widget, wrtype);
			break;
		case TEXT:
			widget->specific.text.set_wrap_type(widget, wrtype);
			break;
		default:
			break;
	}

	_pw_redraw_queue_add(widget, 1);
}

void pw_set_justify(pw_widget widget, unsigned justification)
{
	unsigned jtype = 99999;

	switch(justification) {
		case PW_JUSTIFY_LEFT:
			jtype = JUSTIFY_LEFT;
			break;
		case PW_JUSTIFY_CENTER:
			jtype = JUSTIFY_CENTER;
			break;
		case PW_JUSTIFY_RIGHT:
			jtype = JUSTIFY_RIGHT;
			break;
	}
	
	switch(widget->wtype) {
		case BUTTON:
			widget->specific.button.set_justification(widget, jtype);
			break;
		case CHECKBUTTON:
			widget->specific.checkbutton.set_justification(widget, jtype);
			break;
		case RADIOBUTTON:
			widget->specific.radiobutton.set_justification(widget, jtype);
			break;
		case LABEL:
			widget->specific.label.set_justification(widget, jtype);
			break;
		default:
			break;
	}

	_pw_redraw_queue_add(widget, 1);
}

void pw_set_border(pw_widget widget, unsigned bordertype)
{
	unsigned btype = 99999;

	switch(bordertype) {
		case PW_NO_BORDER:
			btype = NO_BORDER;
			break;
		case PW_RECTANGLE_BORDER:
			btype = RECTANGLE_BORDER;
			break;
		case PW_BEVELLED_BORDER:
			btype = BEVELLED_BORDER;
			break;
		case PW_ELLIPTIC_BORDER:
			btype = ELLIPTIC_BORDER;
			break;
	}
	
	widget->set_border_type(widget, btype);
	_pw_redraw_queue_add(widget, 1);
}

void pw_set_font(pw_widget widget, const char *font)
{
	switch(widget->wtype) {
		case BUTTON:
			widget->specific.button.set_font(widget, font);
			break;
		case CHECKBUTTON:
			widget->specific.checkbutton.set_font(widget, font);
			break;
		case RADIOBUTTON:
			widget->specific.radiobutton.set_font(widget, font);
			break;
		case PROGRESSBAR:
			widget->specific.progressbar.set_font(widget, font);
			break;
		case LABEL:
			widget->specific.label.set_font(widget, font);
			break;
		case ENTRY:
			widget->specific.entry.set_font(widget, font);
			break;
		case TEXT:
			widget->specific.text.set_font(widget, font);
			break;
		case TXTLB:
			txtlb_set_font(widget, font);
			break;
		case VSLIDER:
			widget->specific.vslider.set_font(widget, font);
			break;
		case HSLIDER:
			widget->specific.hslider.set_font(widget, font);
			break;
		case COMBOBOX:
			combobox_set_font(widget, font);
			break;
		default:
			break;
	}

	_pw_redraw_queue_add(widget, 1);
}

void pw_set_normal_image(pw_widget widget, char *filename)
{
	switch(widget->wtype) {
		case BUTTON:
			widget->specific.button.set_normal_pixmap_from_file(widget, filename);
			break;
		case CHECKBUTTON:
			widget->specific.button.set_normal_pixmap_from_file(widget, filename);
			break;
		case RADIOBUTTON:
			widget->specific.button.set_normal_pixmap_from_file(widget, filename);
			break;
		default:
			break;
	}

	_pw_redraw_queue_add(widget, 1);
}

void pw_set_active_image(pw_widget widget, char *filename)
{
	switch(widget->wtype) {
		case BUTTON:
			widget->specific.button.set_active_pixmap_from_file(widget, filename);
			break;
		case CHECKBUTTON:
			widget->specific.button.set_active_pixmap_from_file(widget, filename);
			break;
		case RADIOBUTTON:
			widget->specific.button.set_active_pixmap_from_file(widget, filename);
			break;
		default:
			break;
	}

	_pw_redraw_queue_add(widget, 1);
}

void pw_set_disabled_image(pw_widget widget, char *filename)
{
	switch(widget->wtype) {
		case BUTTON:
			widget->specific.button.set_disabled_pixmap_from_file(widget, filename);
			break;
		case CHECKBUTTON:
			widget->specific.button.set_disabled_pixmap_from_file(widget, filename);
			break;
		case RADIOBUTTON:
			widget->specific.button.set_disabled_pixmap_from_file(widget, filename);
			break;
		default:
			break;
	}

	_pw_redraw_queue_add(widget, 1);
}

void pw_set_tag(pw_widget widget, const char *tag, ...)
{
	char *txt=NULL;
     va_list ap;

     va_start(ap, tag);
     txt = _pw_my_printf(tag, ap);
     va_end(ap);

	if(widget->tag)
		free(widget->tag);
	widget->tag = strdup(txt);

	if(txt)
		free(txt);
}

void pw_focus(pw_widget widget)
{
	px_eventloop *eloop = eventloop_get_eventloop();

	eloop->focus(eloop, widget);
}

void pw_grab(pw_widget widget)
{
	px_eventloop *eloop = eventloop_get_eventloop();

	eloop->grab(eloop, widget);
}

void pw_ungrab()
{
	px_eventloop *eloop = eventloop_get_eventloop();

	eloop->grab(eloop, NULL);
}

void pw_set_callback(pw_widget widget, unsigned eventtype, Bool (*callback)(px_widget*, px_event*))
{
	unsigned etype = 99999;

	if(eventtype==PW_WINDOW_DELETE_EVENT) {
		widget->window_delete_callback = callback;
	} else if(eventtype>=PW_RETRACT_BAR_REL_EVENT && eventtype<=PW_FLICK_EXTREME_LEFT_EVENT) {
		switch(eventtype) {
			case PW_RETRACT_BAR_REL_EVENT:
				etype = RETRACT_BAR_REL;
				break;
			case PW_UP_RELEASE_EVENT:
				etype = UP_REL;
				break;
			case PW_TICK_RELEASE_EVENT:
				etype = TICK_REL;
				break;
			case PW_NEXT_PAGE_EVENT:
				etype = NEXT_PAGE;
				break;
			case PW_NEXT_PAGE_RELEASE_EVENT:
				etype = NEXT_PAGE_REL;
				break;
			case PW_PREV_PAGE_EVENT:
				etype = PREV_PAGE;
				break;
			case PW_PREV_PAGE_RELEASE_EVENT:
				etype = PREV_PAGE_REL;
				break;
			case PW_EMAIL_RELEASE_EVENT:
				etype = EMAIL_REL;
				break;
			case PW_PRINTER_RELEASE_EVENT:
				etype = PRINTER_REL;
				break;
			case PW_ROCKER_UP_EVENT:
				etype = ROCKER_UP;
				break;
			case PW_ROCKER_UP_RELEASE_EVENT:
				etype = ROCKER_UP_REL;
				break;
			case PW_ROCKER_DOWN_EVENT:
				etype = ROCKER_DOWN;
				break;
			case PW_ROCKER_DOWN_RELEASE_EVENT:
				etype = ROCKER_DOWN_REL;
				break;
			case PW_ROCKER_SELECT_EVENT:
				etype = ROCKER_SELECT;
				break;
			case PW_ROCKER_SELECT_RELEASE_EVENT:
				etype = ROCKER_SELECT_REL;
				break;
			case PW_FLICK_RIGHT_EVENT:
				etype = ACCEL_RIGHT;
				break;
			case PW_FLICK_LEFT_EVENT:
				etype = ACCEL_LEFT;
				break;
			case PW_FLICK_TOP_EVENT:
				etype = ACCEL_TOP;
				break;
			case PW_FLICK_BOTTOM_EVENT:
				etype = ACCEL_BOTTOM;
				break;
			case PW_FLICK_EXTREME_RIGHT_EVENT:
				etype = ACCEL_RIGHT_EXTREME;
				break;
			case PW_FLICK_EXTREME_LEFT_EVENT:
				etype = ACCEL_LEFT_EXTREME;
				break;
		}

		widget->extra_callbacks[etype] = callback;
	} else {
		switch(eventtype) {
			case PW_BUTTON_PRESS_EVENT:
				etype = BUTTON_PRESSED;
				break;
			case PW_BUTTON_RELEASE_EVENT:
				etype = BUTTON_RELEASED;
				break;
			case PW_MOTION_EVENT:
				etype = MOTION;
				break;
			case PW_MOUSE_ENTER_EVENT:
				etype = MOUSE_ENTER;
				break;
			case PW_MOUSE_LEAVE_EVENT:
				etype = MOUSE_LEAVE;
				break;
			case PW_KEY_PRESS_EVENT:
				etype = KEY_PRESSED;
				break;
			case PW_KEY_RELEASE_EVENT:
				etype = KEY_RELEASED;
				break;
			case PW_CONFIGURE_EVENT:
				etype = CONFIGURE;
				break;
			case PW_CLIENT_MESSAGE_EVENT:
				etype = CLIENT_MESSAGE;
				break;
			case PW_VISIBILITY_EVENT:
				etype = VISIBILITY;
				break;
		}
		
		widget->callbacks[etype] = callback;
	}
}

void pw_set_callback_parameter(pw_widget widget, unsigned eventtype, void *parameter)
{
	unsigned etype = 99999;

	if(eventtype==PW_WINDOW_DELETE_EVENT) {
		widget->window_delete_callback_parameter = parameter;
	} else if(eventtype>=PW_RETRACT_BAR_REL_EVENT && eventtype<=PW_FLICK_EXTREME_LEFT_EVENT) {
		switch(eventtype) {
			case PW_RETRACT_BAR_REL_EVENT:
				etype = RETRACT_BAR_REL;
				break;
			case PW_UP_RELEASE_EVENT:
				etype = UP_REL;
				break;
			case PW_TICK_RELEASE_EVENT:
				etype = TICK_REL;
				break;
			case PW_NEXT_PAGE_EVENT:
				etype = NEXT_PAGE;
				break;
			case PW_NEXT_PAGE_RELEASE_EVENT:
				etype = NEXT_PAGE_REL;
				break;
			case PW_PREV_PAGE_EVENT:
				etype = PREV_PAGE;
				break;
			case PW_PREV_PAGE_RELEASE_EVENT:
				etype = PREV_PAGE_REL;
				break;
			case PW_EMAIL_RELEASE_EVENT:
				etype = EMAIL_REL;
				break;
			case PW_PRINTER_RELEASE_EVENT:
				etype = PRINTER_REL;
				break;
			case PW_ROCKER_UP_EVENT:
				etype = ROCKER_UP;
				break;
			case PW_ROCKER_UP_RELEASE_EVENT:
				etype = ROCKER_UP_REL;
				break;
			case PW_ROCKER_DOWN_EVENT:
				etype = ROCKER_DOWN;
				break;
			case PW_ROCKER_DOWN_RELEASE_EVENT:
				etype = ROCKER_DOWN_REL;
				break;
			case PW_ROCKER_SELECT_EVENT:
				etype = ROCKER_SELECT;
				break;
			case PW_ROCKER_SELECT_RELEASE_EVENT:
				etype = ROCKER_SELECT_REL;
				break;
			case PW_FLICK_RIGHT_EVENT:
				etype = ACCEL_RIGHT;
				break;
			case PW_FLICK_LEFT_EVENT:
				etype = ACCEL_LEFT;
				break;
			case PW_FLICK_TOP_EVENT:
				etype = ACCEL_TOP;
				break;
			case PW_FLICK_BOTTOM_EVENT:
				etype = ACCEL_BOTTOM;
				break;
			case PW_FLICK_EXTREME_RIGHT_EVENT:
				etype = ACCEL_RIGHT_EXTREME;
				break;
			case PW_FLICK_EXTREME_LEFT_EVENT:
				etype = ACCEL_LEFT_EXTREME;
				break;
		}

		widget->extra_callbacks_parameters[etype] = parameter;
	} else {
		switch(eventtype) {
			case PW_BUTTON_PRESS_EVENT:
				etype = BUTTON_PRESSED;
				break;
			case PW_BUTTON_RELEASE_EVENT:
				etype = BUTTON_RELEASED;
				break;
			case PW_MOTION_EVENT:
				etype = MOTION;
				break;
			case PW_MOUSE_ENTER_EVENT:
				etype = MOUSE_ENTER;
				break;
			case PW_MOUSE_LEAVE_EVENT:
				etype = MOUSE_LEAVE;
				break;
			case PW_KEY_PRESS_EVENT:
				etype = KEY_PRESSED;
				break;
			case PW_KEY_RELEASE_EVENT:
				etype = KEY_RELEASED;
				break;
			case PW_CONFIGURE_EVENT:
				etype = CONFIGURE;
				break;
			case PW_CLIENT_MESSAGE_EVENT:
				etype = CLIENT_MESSAGE;
				break;
			case PW_VISIBILITY_EVENT:
				etype = VISIBILITY;
				break;
		}
		
		widget->callback_parameters[etype] = parameter;
	}
}

void pw_destroy(pw_widget widget)
{
	widget->destruct(widget);
}

void pw_destroy_many(pw_widget widget, ...)
{
	va_list ap;
	pw_widget w;

	if(widget) {
		widget->destruct(widget);
		va_start(ap, widget);
		while(1) {
			w = va_arg(ap, pw_widget);
			if(w)
				w->destruct(w);
			else
				break;
		}
		va_end(ap);
	}
}

void pw_configure(pw_widget widget, ...)
{
	va_list vl;

	va_start(vl, widget);
	_pw_serve_options(widget, vl);
	va_end(vl);
}

void pw_update(pw_widget widget, int show)
{
	widget->redraw(widget);
	if(show)
		widget->show(widget);
}
