/*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
// (C) PicoPeta Simputers Pvt. Ltd. (2003)
// Author : Saugata Das
// Graphics functions
#include "pwrap.h"
#include "pdefs.h"

void pw_set_drawcolor(const char *color)
{
	XColor col;

	widget_xcolor_from_str(&col, color);

	XSetForeground(xdisplay, draw_gc, col.pixel);
}

void pw_draw_point(pw_widget widget, int x, int y)
{
	if(widget->wtype!=FRAME && widget->wtype!=TOPLEVEL)
		return;

	XDrawPoint(xdisplay, widget->backing_store.pxmp, draw_gc, x, y);
     XClearWindow(xdisplay, widget->win);
}

void pw_draw_line(pw_widget widget, int x1, int y1, int x2, int y2)
{
	if(widget->wtype!=FRAME && widget->wtype!=TOPLEVEL)
		return;

	XSetLineAttributes(xdisplay, draw_gc, 1, LineSolid, CapRound, JoinRound);
	XDrawLine(xdisplay, widget->backing_store.pxmp, draw_gc, x1, y1, x2, y2);
     XClearWindow(xdisplay, widget->win);
}

void pw_draw_rectangle(pw_widget widget, int x, int y, int width, int height)
{
	if(widget->wtype!=FRAME && widget->wtype!=TOPLEVEL)
		return;

	XDrawRectangle(xdisplay, widget->backing_store.pxmp, draw_gc, x, y, width, height);
     XClearWindow(xdisplay, widget->win);
}

void pw_fill_rectangle(pw_widget widget, int x, int y, int width, int height)
{
	if(widget->wtype!=FRAME && widget->wtype!=TOPLEVEL)
		return;

	XFillRectangle(xdisplay, widget->backing_store.pxmp, draw_gc, x, y, width, height);
     XClearWindow(xdisplay, widget->win);
}

void pw_draw_circle(pw_widget widget, int x, int y, int radius)
{
	if(widget->wtype!=FRAME && widget->wtype!=TOPLEVEL)
		return;

	XDrawArc(xdisplay, widget->backing_store.pxmp, draw_gc, x-radius, y-radius, 
												2*radius, 2*radius, 0, 360*64);
     XClearWindow(xdisplay, widget->win);
}

void pw_draw_arc(pw_widget widget, int x, int y, int a, int b, int start_angle, int end_angle)
{
	if(widget->wtype!=FRAME && widget->wtype!=TOPLEVEL)
		return;

	XDrawArc(xdisplay, widget->backing_store.pxmp, draw_gc, 
								x-a, y-b, 2*a, 2*b, start_angle*64, end_angle*64);
     XClearWindow(xdisplay, widget->win);
}

void pw_fill_arc(pw_widget widget, int x, int y, int a, int b, int start_angle, int end_angle)
{
	if(widget->wtype!=FRAME && widget->wtype!=TOPLEVEL)
		return;

	XFillArc(xdisplay, widget->backing_store.pxmp, draw_gc,
								x-a, y-b, 2*a, 2*b, start_angle*64, end_angle*64);
     XClearWindow(xdisplay, widget->win);
}
