/*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
// (C) PicoPeta Simputers Pvt. Ltd. (2003)
// Author : Saugata Das
#ifndef 	__PW__
#define  	__PW__

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/keysym.h>

#include <picowidget.h>
#include <pimglib.h>
#include <megawidget.h>
#include <display.h>
#include <image.h>
#include <eventloop.h>
#include <acctrl.h>
#include <txtlb.h>
#include <combobox.h>

#define PW_NORMAL_FONT 	"-picopeta-simputer-medium-r-normal--11-*-*-*-p-*-fontspecific-0"
#define PW_BOLD_FONT 	"-picopeta-simputer-bold-r-normal--11-*-*-*-p-*-fontspecific-0"
#define PW_SMALL_FONT 	"-picopeta-simputer-medium-r-normal--9-*-*-*-p-*-fontspecific-0"
#define PW_TINY_FONT 	"-picopeta-simputer-medium-r-normal--7-*-*-*-p-*-fontspecific-0"
#define PW_LARGE_FONT 	"-picopeta-simputer-medium-r-normal--14-*-*-*-p-*-fontspecific-0"
#define PW_FIXED_FONT	"fixed"

#define PW_SUPPORT_GENERIC		0x00000000
#define PW_SUPPORT_ALCHEMY		0x00000001
#define PW_SUPPORT_ROCKER		0x00000002
#define PW_SUPPORT_ACCELEROMETER	0x00000004
#define PW_SUPPORT_DRAFTMODE		0x00000008

#define PW_BUTTON_PRESS_EVENT			0
#define PW_BUTTON_RELEASE_EVENT		1
#define PW_MOTION_EVENT				2
#define PW_MOUSE_ENTER_EVENT			3
#define PW_MOUSE_LEAVE_EVENT			4
#define PW_KEY_PRESS_EVENT			5
#define PW_KEY_RELEASE_EVENT			6
#define PW_CONFIGURE_EVENT			7
#define PW_CLIENT_MESSAGE_EVENT		8
#define PW_VISIBILITY_EVENT			9
#define PW_WINDOW_DELETE_EVENT		10
#define PW_RETRACT_BAR_REL_EVENT		11
#define PW_UP_RELEASE_EVENT			12
#define PW_TICK_RELEASE_EVENT			13
#define PW_NEXT_PAGE_EVENT			14
#define PW_NEXT_PAGE_RELEASE_EVENT		15
#define PW_PREV_PAGE_EVENT			16
#define PW_PREV_PAGE_RELEASE_EVENT		17
#define PW_EMAIL_RELEASE_EVENT		18
#define PW_PRINTER_RELEASE_EVENT		19
#define PW_ROCKER_UP_EVENT			20
#define PW_ROCKER_UP_RELEASE_EVENT		21
#define PW_ROCKER_DOWN_EVENT			22
#define PW_ROCKER_DOWN_RELEASE_EVENT	23
#define PW_ROCKER_SELECT_EVENT		24
#define PW_ROCKER_SELECT_RELEASE_EVENT	25
#define PW_FLICK_RIGHT_EVENT			26
#define PW_FLICK_LEFT_EVENT			27
#define PW_FLICK_TOP_EVENT			28
#define PW_FLICK_BOTTOM_EVENT			29
#define PW_FLICK_EXTREME_RIGHT_EVENT	30
#define PW_FLICK_EXTREME_LEFT_EVENT	31

#define PW_JUSTIFY_LEFT				0
#define PW_JUSTIFY_CENTER			1
#define PW_JUSTIFY_RIGHT				2

#define PW_NO_BORDER				0
#define PW_RECTANGLE_BORDER			1
#define PW_BEVELLED_BORDER			2
#define PW_ELLIPTIC_BORDER			3

#define PW_NO_WRAP					0
#define PW_CHAR_WRAP				1
#define PW_WORD_WRAP				2

#define PW_ENABLED					0
#define PW_DISABLED					1

#define PW_NA						-1
#define PW_CONFIG_END				-1
#define PW_TITLE					0
#define PW_TEXT					1
#define PW_IMAGE					2
#define PW_FONT					3
#define PW_JUSTIFY					4
#define PW_WRAP					5
#define PW_BORDER					6
#define PW_TAG						7
#define PW_FGCOLOR					8
#define PW_BGCOLOR					9
#define PW_BGTRANSPARENT				10
#define PW_BGIMAGE_FILE				11
#define PW_POSITION					12
#define PW_SIZE					13
#define PW_NORMAL_IMAGE				14
#define PW_ACTIVE_IMAGE				15
#define PW_DISABLED_IMAGE			16
#define PW_STATE					17
#define PW_CALLBACK					18
#define PW_CALLBACK_PARAM			19

// Orientation type, used by pw_set_display_orientation() function.
#define PW_ORIENT_TOP				0
#define PW_ORIENT_BOTTOM				1
#define PW_ORIENT_LEFT				2
#define PW_ORIENT_RIGHT				3

// Slider text print orientation.
#define PW_SLIDER_TOP				0
#define PW_SLIDER_BOTTOM				1
#define PW_SLIDER_LEFT				2
#define PW_SLIDER_RIGHT				3

// Text listbox selection types.
#define PW_TLISTBOX_SINGLE_SELECTION	0
#define PW_TLISTBOX_MULTIPLE_SELECTION	1

// Commands that can be passed in pw_alchemy_command() function.
#define PW_WM_SET_THEME              	AC_CMD_SET_THEME
#define PW_WM_EXIT                   	AC_CMD_EXIT
#define PW_WM_NEXT                   	AC_CMD_NEXT
#define PW_WM_PREV                   	AC_CMD_PREV
#define PW_WM_SET_FLAGS              	AC_CMD_SET_FLAGS
#define PW_WM_LAUNCH_APP             	AC_CMD_LAUNCH_APP
#define PW_WM_SET_TITLE_FONT         	AC_CMD_SET_TITLE_FONT
#define PW_WM_SET_TITLE_JUSTIFY      	AC_CMD_SET_TITLE_JUSTIFY
#define PW_WM_SET_STATUS_MSG         	AC_CMD_SET_STATUS_MSG
#define PW_WM_SET_STATUS_FONT        	AC_CMD_SET_STATUS_FONT
#define PW_WM_SET_STATUS_JUSTIFY     	AC_CMD_SET_STATUS_JUSTIFY
#define PW_WM_SET_PAGE_NUM           	AC_CMD_SET_PAGE_NUM
#define PW_WM_SET_PAGE_NUM_FONT      	AC_CMD_SET_PAGE_NUM_FONT
#define PW_WM_SET_PAGE_NUM_JUSTIFY   	AC_CMD_SET_PAGE_NUM_JUSTIFY
#define PW_WM_SET_KEYBOARD_LAYOUT    	AC_CMD_SET_KEYBOARD_LAYOUT
#define PW_WM_SET_KEYBOARD_CATEGORY  	AC_CMD_SET_KEYBOARD_CATEGORY
#define PW_WM_SET_ANNOTATE_KEY       	AC_CMD_SET_ANNOTATE_KEY
#define PW_WM_SET_HELP_KEY           	AC_CMD_SET_HELP_KEY
#define PW_WM_SET_EMAIL_FILE         	AC_CMD_SET_EMAIL_FILE
#define PW_WM_DEL_ANNOTATION         	AC_CMD_DEL_ANNOTATION
#define PW_WM_SET_ROTATION           	AC_CMD_SET_ROTATION
#define PW_WM_LAYER_NEXT_APP         	AC_CMD_LAYER_NEXT_APP
#define PW_WM_LAYER_THIS_WIN         	AC_CMD_LAYER_THIS_WIN
#define PW_WM_SHOW_INPUT             	AC_CMD_SHOW_INPUT
#define PW_WM_HIDE_INPUT             	AC_CMD_HIDE_INPUT
#define PW_WM_SHOW_ANNOTATE          	AC_CMD_SHOW_ANNOTATE
#define PW_WM_HIDE_ANNOTATE          	AC_CMD_HIDE_ANNOTATE
#define PW_WM_SHOW_EMAIL             	AC_CMD_SHOW_EMAIL
#define PW_WM_HIDE_EMAIL             	AC_CMD_HIDE_EMAIL
#define PW_WM_SHOW_PRINTER           	AC_CMD_SHOW_PRINTER
#define PW_WM_HIDE_PRINTER           	AC_CMD_HIDE_PRINTER
#define PW_WM_SHOW_PAGE_NAV          	AC_CMD_SHOW_PAGE_NAV
#define PW_WM_HIDE_PAGE_NAV          	AC_CMD_HIDE_PAGE_NAV
#define PW_WM_ENABLE_PAGE_NEXT       	AC_CMD_ENABLE_PAGE_NEXT
#define PW_WM_DISABLE_PAGE_NEXT      	AC_CMD_DISABLE_PAGE_NEXT
#define PW_WM_ENABLE_PAGE_PREV       	AC_CMD_ENABLE_PAGE_PREV
#define PW_WM_DISABLE_PAGE_PREV      	AC_CMD_DISABLE_PAGE_PREV
#define PW_WM_SHOW_RETRACTBAR        	AC_CMD_SHOW_RETRACTBAR
#define PW_WM_HIDE_RETRACTBAR        	AC_CMD_HIDE_RETRACTBAR
#define PW_WM_SHOW_KEYBOARD          	AC_CMD_SHOW_KEYBOARD
#define PW_WM_HIDE_KEYBOARD          	AC_CMD_HIDE_KEYBOARD
#define PW_WM_SHOW_STATUS            	AC_CMD_SHOW_STATUS
#define PW_WM_HIDE_STATUS            	AC_CMD_HIDE_STATUS
#define PW_WM_SWAP_CLOSE_UP          	AC_CMD_SWAP_CLOSE_UP
#define PW_WM_SHOW_CLOSE             	AC_CMD_SHOW_CLOSE
#define PW_WM_SHOW_UP                	AC_CMD_SHOW_UP
#define PW_WM_PRINTER_STATUS         	AC_CMD_PRINTER_STATUS
#define PW_WM_SHOW_SPEAKER           	AC_CMD_SHOW_SPEAKER
#define PW_WM_HIDE_SPEAKER           	AC_CMD_HIDE_SPEAKER
#define PW_WM_SHOW_VOICE             	AC_CMD_SHOW_VOICE
#define PW_WM_HIDE_VOICE             	AC_CMD_HIDE_VOICE
#define PW_WM_APP_SPEAKS             	AC_CMD_APP_SPEAKS
#define PW_WM_AC_SPEAKS              	AC_CMD_AC_SPEAKS
#define PW_WM_SHOW_TICK              	AC_CMD_SHOW_TICK
#define PW_WM_SHOW_HELP              	AC_CMD_SHOW_HELP
#define PW_WM_HIDE_HELP              	AC_CMD_HIDE_HELP
#define PW_WM_GRAB_ON                	AC_CMD_GRAB_ON
#define PW_WM_GRAB_OFF               	AC_CMD_GRAB_OFF

// MessageBox button, return and icon types.
#define PW_MSGBOX_BUTTON_YESNO	    		MSGTYPE_YESNO
#define PW_MSGBOX_BUTTON_YESNOCANCEL		MSGTYPE_YESNOCANCEL
#define PW_MSGBOX_BUTTON_OK				MSGTYPE_OK
#define PW_MSGBOX_BUTTON_OKCANCEL			MSGTYPE_OKCANCEL
#define PW_MSGBOX_BUTTON_RETRYCANCEL		MSGTYPE_RETRYCANCEL
#define PW_MSGBOX_BUTTON_ABORTRETRYCANCEL	MSGTYPE_ABORTRETRYCANCEL

#define PW_MSGBOX_RETURN_YES			    MSGBTN_YES
#define PW_MSGBOX_RETURN_NO			    MSGBTN_NO
#define PW_MSGBOX_RETURN_CANCEL		    MSGBTN_CANCEL
#define PW_MSGBOX_RETURN_OK			    MSGBTN_OK
#define PW_MSGBOX_RETURN_RETRY		    MSGBTN_RETRY
#define PW_MSGBOX_RETURN_ABORT		    MSGBTN_ABORT
#define PW_MSGBOX_RETURN_NOBUTTON		    MSGBTN_NOBUTTON

#define PW_MSGBOX_ICON_ERROR			    MSGICON_ERROR
#define PW_MSGBOX_ICON_INFO			    MSGICON_INFO
#define PW_MSGBOX_ICON_QUESTION		    MSGICON_QUESTION
#define PW_MSGBOX_ICON_WARNING		    MSGICON_WARNING
#define PW_MSGBOX_ICON_NOICON			    MSGICON_NOICON

typedef px_widget* pw_widget;
typedef px_timer* pw_timer;
typedef px_event* pw_event;
typedef px_pixmap* pw_pixmap;
typedef struct Image* pw_image;

////// [BASIC]
int pw_init(unsigned flags);
void pw_eventloop();
void pw_exit();

////// [EVENT HANDLING]
void pw_stop_eventloop();
void pw_handle_one_event();
void pw_flush_eventloop();
pw_timer pw_create_timer(unsigned long time, int repeat, Bool (*callback)(void*, void*), void *arg1, void *arg2);
void pw_destroy_timer(pw_timer timer);
int pw_event_getx(pw_event event);
int pw_event_gety(pw_event event);
unsigned char pw_event_getchar(pw_event event);
void *pw_event_callback_parameter(pw_event event);

////// [IMAGE]
pw_pixmap pw_create_pixmap_from_file(const char *file, ...);
void pw_destroy_pixmap(pw_pixmap pixmap);
int pw_get_pixmap_width(pw_pixmap pixmap);
int pw_get_pixmap_height(pw_pixmap pixmap);
void pw_draw_pixmap(pw_pixmap src_pixmap, pw_widget dest_widget,  
				int src_x, int src_y, int width, int height, int dest_x, int dest_y);

////// [GENERIC]
void pw_show(pw_widget widget);
void pw_show_many(pw_widget widget, ...);
void pw_hide(pw_widget widget);
void pw_hide_many(pw_widget widget, ...);
void pw_raise(pw_widget widget);
void pw_lower(pw_widget widget);
void pw_enable(pw_widget widget);
void pw_enable_many(pw_widget widget, ...);
void pw_disable(pw_widget widget);
void pw_disable_many(pw_widget widget, ...);
void pw_destroy(pw_widget widget);
void pw_destroy_many(pw_widget widget, ...);
void pw_focus(pw_widget widget);
void pw_grab(pw_widget widget);
void pw_ungrab();
void pw_configure(pw_widget widget, ...);
void pw_update(pw_widget widget, int show);

void pw_set_title(pw_widget widget, const char *title);
void pw_set_text(pw_widget widget, const char *text, ...);
void pw_set_font(pw_widget widget, const char *font);
void pw_set_justify(pw_widget widget, unsigned justification);
void pw_set_wrap(pw_widget widget, unsigned wraptype);
void pw_set_border(pw_widget widget, unsigned bordertype);
void pw_set_tag(pw_widget widget, const char *tag, ...);
void pw_set_fgcolor(pw_widget widget, const char *color);
void pw_set_bgcolor(pw_widget widget, const char *color);
void pw_set_bgtransparent(pw_widget widget);
void pw_set_bgimage_file(pw_widget widget, const char *file, ...);
void pw_set_position(pw_widget widget, int x, int y);
void pw_set_size(pw_widget widget, unsigned long width, unsigned long height);
void pw_set_normal_image(pw_widget widget, char *filename);
void pw_set_active_image(pw_widget widget, char *filename);
void pw_set_disabled_image(pw_widget widget, char *filename);
void pw_set_callback(pw_widget widget, unsigned eventtype, Bool (*callback)(px_widget*, px_event*));
void pw_set_callback_parameter(pw_widget widget, unsigned eventtype, void *parameter);

void pw_get_position(pw_widget widget, int *x, int *y);
const char *pw_get_text(pw_widget widget);
const char *pw_get_tag(pw_widget widget);
void pw_get_size(pw_widget widget, unsigned long *width, unsigned long *height);
	
////// [GRAPHICS]
void pw_set_drawcolor(const char *color);
void pw_draw_point(pw_widget widget, int x, int y);
void pw_draw_line(pw_widget widget, int x1, int y1, int x2, int y2);
void pw_draw_rectangle(pw_widget widget, int x, int y, int width, int height);
void pw_fill_rectangle(pw_widget widget, int x, int y, int width, int height);
void pw_draw_circle(pw_widget widget, int x, int y, int radius);
void pw_draw_arc(pw_widget widget, int x, int y, int a, int b, int start_angle, int end_angle);
void pw_fill_arc(pw_widget widget, int x, int y, int a, int b, int start_angle, int end_angle);

////// [EXTRAS]
void pw_alchemy_command(pw_widget toplevel, int command, const char *data, ...);
messagebox_button pw_messagebox(	pw_widget toplevel,
							messagebox_button type, messagebox_icon icon,
							const char *title, const char *mesg
						);
void pw_set_display_orientation(pw_widget window, unsigned orientation);
unsigned pw_get_display_width(pw_widget window);
unsigned pw_get_display_height(pw_widget window);
unsigned pw_get_display_orientation(pw_widget window);
void pw_toggle_fullscreen(pw_widget window);

////// [WIDGETS]
/// [WINDOW]
pw_widget pw_create_window(unsigned width, unsigned height, ...);
pw_widget pw_create_child_window(pw_widget parent, unsigned width, unsigned height, ...);
int pw_compare_clientmessage(px_event *event, const char *message);
int pw_configure_event_width(px_event *event);
int pw_configure_event_height(px_event *event);

/// [DIALOG WINDOW]
pw_widget pw_create_dialog(unsigned width, unsigned height, ...);
pw_widget pw_create_child_dialog(pw_widget parent, unsigned width, unsigned height, ...);

/// [FRAME]
pw_widget pw_create_frame(	pw_widget parent,
						int x, int y, unsigned width, unsigned height, ...);

/// [LABEL]
pw_widget pw_create_label(	pw_widget parent,
						int x, int y, unsigned width, unsigned height, ...);

/// [BUTTON]
pw_widget pw_create_button(	pw_widget parent,
                              int x, int y, unsigned width, unsigned height, ...);

/// [ENTRY]
pw_widget pw_create_entry(	pw_widget parent,
                              int x, int y, unsigned width, unsigned height, ...);
void pw_entry_clear(pw_widget widget);
void pw_entry_passwdchar(pw_widget widget, char ch);

/// [RADIO BUTTON]
pw_widget pw_create_radiobutton(	pw_widget parent,
							int x, int y, unsigned width, unsigned height, 
							pw_widget group, ...);
void pw_radiobutton_select(pw_widget widget);
unsigned pw_radiobutton_status(pw_widget widget);

/// [CHECK BUTTON]
pw_widget pw_create_checkbutton(	pw_widget parent,
	                              int x, int y, unsigned width, unsigned height, ...);
void pw_checkbutton_select(pw_widget widget);
void pw_checkbutton_deselect(pw_widget widget);
unsigned pw_checkbutton_status(pw_widget widget);

/// [WIDGET LISTBOX]
pw_widget pw_create_wlistbox(	pw_widget parent,
                              int x, int y, unsigned width, unsigned height, ...);
void pw_wlistbox_insert(pw_widget widget, pw_widget element, unsigned index);
void pw_wlistbox_insertend(pw_widget widget, pw_widget element);
void pw_wlistbox_remove(pw_widget widget, unsigned rownum);
void pw_wlistbox_clear(pw_widget widget);
pw_widget pw_wlistbox_get_element(pw_widget widget, unsigned rownum);
int pw_wlistbox_get_selection(pw_widget widget);
void pw_wlistbox_see(pw_widget widget, unsigned rownum);
int pw_wlistbox_get_numrows(pw_widget widget);

/// [TEXT LISTBOX]
pw_widget pw_create_tlistbox(	pw_widget parent,
                              int x, int y, unsigned width, unsigned height, ...);
void pw_tlistbox_addcolumn(pw_widget widget, unsigned width);
void pw_tlistbox_insert(pw_widget widget, char *text, unsigned rownum, unsigned colnum);
void pw_tlistbox_insertend(pw_widget widget, char *text, unsigned colnum);
void pw_tlistbox_remove(pw_widget widget, unsigned row, Bool refresh_fg);
void pw_tlistbox_clear(pw_widget widget);
void pw_tlistbox_change(pw_widget widget, char *text, unsigned rownum, unsigned colnum);
int pw_tlistbox_get_selection(pw_widget widget);
int pw_tlistbox_get_multiple_selection(pw_widget widget, int **selection_index);
void pw_tlistbox_set_selection(pw_widget widget, unsigned rownum);
void pw_tlistbox_get_element(pw_widget widget, char **text, unsigned rownum, unsigned colnum);
int pw_tlistbox_get_numrows(pw_widget widget);
void pw_tlistbox_see(pw_widget widget, unsigned rownum);
void pw_tlistbox_set_selection_type(pw_widget widget, unsigned selection_type);
void pw_tlistbox_selection_intimate(pw_widget widget, 
								void (*change_fn)(pw_widget widget, int rownum));

/// [PROGRESS BAR]
pw_widget pw_create_progressbar(	pw_widget parent,
                              	int x, int y, unsigned width, unsigned height, ...);
void pw_progressbar_setvalue(pw_widget widget, unsigned value);
void pw_progressbar_setcolor(pw_widget widget, const char *color);
void pw_progressbar_showvalue(pw_widget widget);
void pw_progressbar_hidevalue(pw_widget widget);

/// [COMBO BOX]
pw_widget pw_create_combobox( pw_widget parent,
							int x, int y, unsigned width, unsigned height, ...);
int pw_combobox_insert(pw_widget widget, char *text, Bool refresh_fg);
int pw_combobox_remove(pw_widget widget, unsigned index);
int pw_combobox_clear(pw_widget widget);
void pw_combobox_get_selection(pw_widget widget, char **text, unsigned *index);
void pw_combobox_set_selection(pw_widget widget, unsigned index);
void pw_combobox_selection_intimate(pw_widget widget,
								void (*change_fn)(pw_widget widget, int index));

/// [TAB]
pw_widget pw_create_tab(	pw_widget parent,
                                   int x, int y, unsigned width, unsigned height, ...);
pw_widget pw_tab_add_page(pw_widget widget, const char *title, Bool(*tab_callback)(pw_widget, pw_event));
void pw_tab_raise_page(pw_widget widget, unsigned tabnum);
pw_widget pw_tab_get_page(pw_widget widget, unsigned tabnum);
pw_widget pw_tab_get_active_page(pw_widget widget);

/// [TEXT BOX]
pw_widget pw_create_textbox(	pw_widget parent,
                                   int x, int y, unsigned width, unsigned height, ...);
void pw_textbox_loadfile(pw_widget widget, const char *filename);
void pw_textbox_savefile(pw_widget widget, const char *filename);
int pw_textbox_get_numlines(pw_widget widget);
const char *pw_textbox_get_line(pw_widget widget, unsigned linenum);

/// [SKETCH]
pw_widget pw_create_sketch(	pw_widget parent,
                                   int x, int y, unsigned width, unsigned height, ...);
void pw_sketch_loadfile(pw_widget widget, const char *filename);
void pw_sketch_savefile(pw_widget widget, const char *filename);
void pw_sketch_pencolor(pw_widget widget, const char *color);
void pw_sketch_penwidth(pw_widget widget, unsigned width);
void pw_sketch_clear(pw_widget widget);

/// [VERTICAL SCROLLBAR]
pw_widget pw_create_vscrollbar(	pw_widget parent,
                                   int x, int y, unsigned width, unsigned height, ...);
void pw_vscrollbar_set_sliderpos(pw_widget widget, float position);
void pw_vscrollbar_set_sliderlen(pw_widget widget, float length);
void pw_vscrollbar_set_sliderincr(pw_widget widget, float increment);
void pw_vscrollbar_intimate_change(pw_widget widget, void (*callback)(px_widget *widget, float position));
float pw_vscrollbar_get_sliderpos(pw_widget widget);
float pw_vscrollbar_get_sliderincr(pw_widget widget);

/// [HORIZONTAL SCROLLBAR]
pw_widget pw_create_hscrollbar(	pw_widget parent,
                                   int x, int y, unsigned width, unsigned height, ...);
void pw_hscrollbar_set_sliderpos(pw_widget widget, float position);
void pw_hscrollbar_set_sliderlen(pw_widget widget, float length);
void pw_hscrollbar_set_sliderincr(pw_widget widget, float increment);
void pw_hscrollbar_intimate_change(pw_widget widget, void (*callback)(px_widget *widget, float position));
float pw_hscrollbar_get_sliderpos(pw_widget widget);
float pw_hscrollbar_get_sliderincr(pw_widget widget);

/// [VERTICAL SLIDER]
pw_widget pw_create_vslider(	pw_widget parent,
                                   int x, int y, unsigned width, unsigned height, ...);
void pw_vslider_set_range(pw_widget widget, int min, int max);
void pw_vslider_set_position(pw_widget widget, int position);
int pw_vslider_get_position(pw_widget widget, int position);
void pw_vslider_show_value(pw_widget widget, unsigned calibration_side);
void pw_vslider_hide_value(pw_widget widget);
void pw_vslider_intimate_release(pw_widget widget, void (*callback)(int position));
void pw_vslider_intimate_motion(pw_widget widget, void (*callback)(int position));

/// [HORIZONTAL SLIDER]
pw_widget pw_create_hslider(	pw_widget parent,
                                   int x, int y, unsigned width, unsigned height, ...);
void pw_hslider_set_range(pw_widget widget, int min, int max);
void pw_hslider_set_position(pw_widget widget, int position);
int pw_hslider_get_position(pw_widget widget, int position);
void pw_hslider_show_value(pw_widget widget, unsigned calibration_side);
void pw_hslider_hide_value(pw_widget widget);
void pw_hslider_intimate_release(pw_widget widget, void (*callback)(int position));
void pw_hslider_intimate_motion(pw_widget widget, void (*callback)(int position));

#endif
