/*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
// (C) PicoPeta Simputers Pvt. Ltd. (2003)
// Author : Saugata Das
// Utility functions for pwrap

#include "pwrap.h"
#include "pdefs.h"

void _pw_serve_options(pw_widget widget, va_list vl)
{
	int fg = 1;
	int option;

	while(fg) {
		option = va_arg(vl, int);
		
		switch(option) {
			case PW_TITLE:
				{
					char *title = va_arg(vl, char*);

					pw_set_title(widget, title);
				}
				break;
			case PW_TEXT:
				{
					char *text = va_arg(vl, char*);

					pw_set_text(widget, text);
				}
				break;
			case PW_FONT:
				{
					char *font = va_arg(vl, char*);

					pw_set_font(widget, font);
				}
				break;
			case PW_JUSTIFY:
				{
					unsigned justification = va_arg(vl, unsigned);

					pw_set_justify(widget, justification);
				}
				break;
			case PW_WRAP:
				{
					unsigned wrap = va_arg(vl, unsigned);

					pw_set_wrap(widget, wrap);
				}
				break;
			case PW_BORDER:
				{
					unsigned border = va_arg(vl, unsigned);

					pw_set_border(widget, border);
				}
				break;
			case PW_TAG:
				{
					char *tag = va_arg(vl, char*);

					pw_set_tag(widget, tag);
				}
				break;
			case PW_FGCOLOR:
				{
					char *color = va_arg(vl, char*);

					pw_set_fgcolor(widget, color);
				}
				break;
			case PW_BGCOLOR:
				{
					char *color = va_arg(vl, char*);

					pw_set_bgcolor(widget, color);
				}
				break;
			case PW_BGTRANSPARENT:
				{
					pw_set_bgtransparent(widget);
				}
				break;
			case PW_BGIMAGE_FILE:
				{
					char *file = va_arg(vl, char*);

					pw_set_bgimage_file(widget, file);
				}
				break;
			case PW_POSITION:
				{
					int x = va_arg(vl, int);
					int y = va_arg(vl, int);

					pw_set_position(widget, x, y);
				}
				break;
			case PW_SIZE:
				{
					int width = va_arg(vl, int);
					int height = va_arg(vl, int);

					pw_set_size(widget, width, height);
				}
				break;
			case PW_NORMAL_IMAGE:
				{
					char *file = va_arg(vl, char*);

					pw_set_normal_image(widget, file);
				}
				break;
			case PW_ACTIVE_IMAGE:
				{
					char *file = va_arg(vl, char*);

					pw_set_active_image(widget, file);
				}
				break;
			case PW_DISABLED_IMAGE:
				{
					char *file = va_arg(vl, char*);

					pw_set_disabled_image(widget, file);
				}
				break;
			case PW_STATE:
				{
					int state = va_arg(vl, int);

					if(state==PW_ENABLED)
						pw_enable(widget);

					if(state==PW_DISABLED)
						pw_disable(widget);
				}
				break;
			case PW_CALLBACK:
				{
					unsigned eventtype = va_arg(vl, unsigned);
					Bool (*callback)(px_widget*, px_event*) = va_arg(vl, void*);

					pw_set_callback(widget, eventtype, callback);
				}
				break;
			case PW_CALLBACK_PARAM:
				{
					unsigned eventtype = va_arg(vl, unsigned);
					void *arg = va_arg(vl, void*);

					pw_set_callback_parameter(widget, eventtype, arg);
				}
				break;
			case PW_IMAGE:
				{
					char *file = va_arg(vl, char*);
					
					switch(widget->wtype) {
						case BUTTON:
							widget->specific.button.set_normal_pixmap_from_file(widget, file);
							break;
						case RADIOBUTTON:
							widget->specific.radiobutton.set_normal_pixmap_from_file(widget, file);
							break;
						case CHECKBUTTON:
							widget->specific.checkbutton.set_normal_pixmap_from_file(widget, file);
							break;
						default:
							pw_set_bgimage_file(widget, file);
							break;
					}
				}
				break;
			case PW_CONFIG_END:
				fg = 0;
				break;
			default:
				break;
		}
	}
}

char *_pw_my_printf(const char *fmt, va_list vl)
{
	int n, size=100;
	va_list vc, vi;
	char *p;

	va_copy(vc, vl);

	if((p=(char*)malloc(size)) == NULL)
		return NULL;

	while(1) {
		va_copy(vi, vc);
		n = vsnprintf(p, size, fmt, vi);
		va_end(vi);	

		if(n>-1 && n<size)
			return p;

		if(n>-1)
			size = n+1; 
		else
			size *= 2;
			
		if((p=(char*)realloc(p,size))==NULL)
			return NULL;
	}

	va_end(vc);

	return p;
}

void _pw_redraw_queue_add(pw_widget widget, int show)
{
#if 1
	px_eventloop *eloop = eventloop_get_eventloop();

	if(widget->wtype == FRAME || widget->wtype == TOPLEVEL) {
		widget->redraw(widget);
		if(widget->hidden==False && show)
			widget->show(widget);
	} else {
		eloop->redraw_queue_add(eloop, widget, show);
	}
#else
	widget->redraw(widget);
	if(widget->hidden==False && show)
		widget->show(widget);
#endif
}
