/*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
// (C) PicoPeta Simputers Pvt. Ltd. (2003)
// Author : Saugata Das
// All Widgets
#include "pwrap.h"
#include "pdefs.h"

/// [TOPLEVEL]
pw_widget pw_create_window(unsigned width, unsigned height, ...)
{
     pw_widget widget;
	va_list vl;

	if(!pdisplay) {
		fprintf(stderr, "pw_init() function not called yet!\n");
		return NULL;
	}
	
	widget = toplevel_win = new_widget(TOPLEVEL, NULL, 0, 0, width, height, NULL);

	// Before mapping, process few pw_init flags
	if(pw_flags&PW_SUPPORT_ALCHEMY) {
		ac_init_app(display_get_Xdisplay(), widget->win);
	}
	
	va_start(vl, height);
	_pw_serve_options(widget, vl);
	va_end(vl);
	
     widget->redraw(widget);
     widget->show(widget);

	// Create GC for graphics and set default graphics settings
	draw_gc = XCreateGC(xdisplay, widget->win, 0, NULL);
	pw_set_drawcolor("black");

	// Draw the grid, if required
	if(pw_flags&PW_SUPPORT_DRAFTMODE) {
		int i;
		XSetFunction(xdisplay, draw_gc, GXxor);
		pw_set_drawcolor("#222222");

		for(i=0; i<=width; i+=10) {
			if(i==0)
				pw_draw_line(widget, i, 0, i, height);
			else
				pw_draw_line(widget, i-1, 0, i-1, height);
		}
		for(i=0; i<=height; i+=10) {
			if(i==0)
				pw_draw_line(widget, 0, i, width, i);
			else
				pw_draw_line(widget, 0, i-1, width, i-1);
		}
		XSetFunction(xdisplay, draw_gc, GXcopy);
	}
	
     return widget;
}

pw_widget pw_create_child_window(pw_widget parent, unsigned width, unsigned height, ...)
{
	if(parent && parent->wtype==TOPLEVEL) {
	     pw_widget widget;
		va_list vl;
		
		widget = new_widget(TOPLEVEL, parent, 0, 0, width, height, NULL);
		
		va_start(vl, height);
		_pw_serve_options(widget, vl);
		va_end(vl);
	
		widget->redraw(widget);
		widget->show(widget);

		return widget;
	} else {
		return NULL;
	}
}

int pw_compare_clientmessage(px_event *event, const char *message)
{
	if(event->xev->xclient.data.l[0] == display_get_atom(pdisplay, message))
		return 1;
	else
		return 0;
}

int pw_configure_event_width(px_event *event)
{
	return event->xev->xconfigure.width;
}

int pw_configure_event_height(px_event *event)
{
	return event->xev->xconfigure.height;
}

/// [DIALOG WINDOW]
pw_widget pw_create_dialog(unsigned width, unsigned height, ...)
{
     pw_widget widget;
	va_list vl;
	Atom old, new;

	if(!pdisplay) {
		fprintf(stderr, "pw_init() function not called yet!\n");
		return NULL;
	}

	old = XInternAtom(xdisplay, "_NET_WM_WINDOW_TYPE", False);
	new = XInternAtom(xdisplay, "_NET_WM_WINDOW_TYPE_DIALOG", False);
		
	widget = toplevel_win = new_widget(TOPLEVEL, NULL, 0, 0, width, height, NULL);

	// Make the widget transient
	XChangeProperty(xdisplay, widget->win, old, XA_ATOM, 32, PropModeReplace, (unsigned char*)&new, 1);
	
	// Before mapping, process few pw_init flags
	if(pw_flags&PW_SUPPORT_ALCHEMY) {
		ac_init_app(display_get_Xdisplay(), widget->win);
	}
	
	va_start(vl, height);
	_pw_serve_options(widget, vl);
	va_end(vl);
	
     widget->redraw(widget);
     widget->show(widget);

	// Create GC for graphics and set default graphics settings
	draw_gc = XCreateGC(xdisplay, widget->win, 0, NULL);
	pw_set_drawcolor("black");

	// Draw the grid, if required
	if(pw_flags&PW_SUPPORT_DRAFTMODE) {
		int i;
		XSetFunction(xdisplay, draw_gc, GXxor);
		pw_set_drawcolor("#222222");

		for(i=0; i<=width; i+=10) {
			if(i==0)
				pw_draw_line(widget, i, 0, i, height);
			else
				pw_draw_line(widget, i-1, 0, i-1, height);
		}
		for(i=0; i<=height; i+=10) {
			if(i==0)
				pw_draw_line(widget, 0, i, width, i);
			else
				pw_draw_line(widget, 0, i-1, width, i-1);
		}
		XSetFunction(xdisplay, draw_gc, GXcopy);
	}

     return widget;
}

pw_widget pw_create_child_dialog(pw_widget parent, unsigned width, unsigned height, ...)
{
	if(parent && parent->wtype==TOPLEVEL) {
	     pw_widget widget;
		va_list vl;
		Atom old, new;
		
		old = XInternAtom(xdisplay, "_NET_WM_WINDOW_TYPE", False);
		new = XInternAtom(xdisplay, "_NET_WM_WINDOW_TYPE_DIALOG", False);
		
		widget = new_widget(TOPLEVEL, parent, 0, 0, width, height, NULL);
		
		// Make the widget transient
		XChangeProperty(xdisplay, widget->win, old, XA_ATOM, 32, PropModeReplace, (unsigned char*)&new, 1);
	
		va_start(vl, height);
		_pw_serve_options(widget, vl);
		va_end(vl);
	
		widget->redraw(widget);
		widget->show(widget);

		return widget;
	} else {
		return NULL;
	}
}

/// [FRAME]
pw_widget pw_create_frame(	pw_widget parent,
						int x, int y, unsigned width, unsigned height, ...)
{
	pw_widget widget = NULL;
	va_list vl;

	widget = new_widget(FRAME, parent, x, y, width, height, "");

	va_start(vl, height);
	_pw_serve_options(widget, vl);
	va_end(vl);
	
	widget->redraw(widget);
	widget->show(widget);
	
	// Draw the grid, if required
	if(pw_flags&PW_SUPPORT_DRAFTMODE) {
		int i;
		XSetFunction(xdisplay, draw_gc, GXxor);
		pw_set_drawcolor("#555555");

		for(i=0; i<=width; i+=10) {
			if(i==0)
				pw_draw_line(widget, i, 0, i, height);
			else
				pw_draw_line(widget, i-1, 0, i-1, height);
		}
		for(i=0; i<=height; i+=10) {
			if(i==0)
				pw_draw_line(widget, 0, i, width, i);
			else
				pw_draw_line(widget, 0, i-1, width, i-1);
		}
		XSetFunction(xdisplay, draw_gc, GXcopy);
	}
	
	return widget;
}

/// [LABEL]
pw_widget pw_create_label(	pw_widget parent,
						int x, int y, unsigned width, unsigned height, ...)
{
	pw_widget widget = NULL;
	px_color col;
	va_list vl;

	col.type = SOLID;
	
	widget = new_widget(LABEL, parent, x, y, width, height, "");

	_pw_redraw_queue_add(widget, 1);
	
	widget_xcolor_from_str(&col.first_color, "white");
	widget->set_background_color(widget, &col);
	widget->specific.label.set_font(widget, PW_NORMAL_FONT);	
	widget->specific.label.set_justification(widget, JUSTIFY_LEFT);
	widget->specific.label.set_wrap_type(widget, NO_WRAP);
	
	widget->redraw(widget);

	va_start(vl, height);
	_pw_serve_options(widget, vl);
	va_end(vl);
	
	return widget;
}

/// [BUTTON]
pw_widget pw_create_button(	pw_widget parent,
                              int x, int y, unsigned width, unsigned height, ...)
{
	pw_widget widget = NULL;
	px_color col;
	va_list vl;

	col.type = SOLID;

	widget = new_widget(BUTTON, parent, x, y, width, height, "");
	widget->redraw(widget);

	_pw_redraw_queue_add(widget, 1);
	
	widget->specific.button.set_font(widget, PW_NORMAL_FONT);
	widget->set_border_type(widget, BEVELLED_BORDER);
	widget_xcolor_from_str(&col.first_color, "white");
	widget->set_background_color(widget, &col);
	widget_xcolor_from_str(&col.first_color, "black");
	widget->specific.button.set_active_background_color(widget, &col);
	widget->specific.button.set_active_font_color(widget, "white");
	widget->specific.button.set_disabled_font_color(widget, "grey");
	widget->specific.button.set_active_border_color(widget, "black");
	widget->specific.button.set_disabled_border_color(widget, "grey");

	va_start(vl, height);
	_pw_serve_options(widget, vl);
	va_end(vl);
	
	return widget;
}

/// [ENTRY]
pw_widget pw_create_entry(	pw_widget parent,
                              int x, int y, unsigned width, unsigned height, ...)
{
	pw_widget widget = NULL;
	va_list vl;

	widget = new_widget(ENTRY, parent, x, y, width, height, "");
	widget->redraw(widget);

	_pw_redraw_queue_add(widget, 1);
	
	va_start(vl, height);
	_pw_serve_options(widget, vl);
	va_end(vl);
	
	return widget;
}

void pw_entry_clear(pw_widget widget)
{
	widget->specific.entry.delete_text(widget, 0, strlen(widget->specific.entry.text));
	_pw_redraw_queue_add(widget, 1);
}

void pw_entry_passwdchar(pw_widget widget, char ch)
{
	widget->specific.entry.password_char = ch;
	_pw_redraw_queue_add(widget, 1);
}

/// [RADIO BUTTON]
pw_widget pw_create_radiobutton(	pw_widget parent,
							int x, int y, unsigned width, unsigned height, 
							pw_widget group, ...)
{
	pw_widget widget = NULL;
	va_list vl;

	widget = new_widget(RADIOBUTTON, parent, x, y, width, height, "");
	widget->redraw(widget);

	_pw_redraw_queue_add(widget, 1);
	
	widget->specific.radiobutton.set_font(widget, PW_NORMAL_FONT);
	widget->specific.radiobutton.set_justification(widget, JUSTIFY_LEFT);
	widget->specific.radiobutton.set_disabled_font_color(widget, "grey");
	if(group)
		widget->specific.radiobutton.group_together(widget, group);

	va_start(vl, group);
	_pw_serve_options(widget, vl);
	va_end(vl);
	
	return widget;
}

void pw_radiobutton_select(pw_widget widget)
{
	widget->specific.radiobutton.select(widget);
	_pw_redraw_queue_add(widget, 1);
}

void pw_radiobutton_deselect(pw_widget widget)
{
	widget->specific.radiobutton.deselect(widget);
	_pw_redraw_queue_add(widget, 1);
}

unsigned pw_radiobutton_status(pw_widget widget)
{
	if(widget->specific.radiobutton.value)
		return 1;
	else
		return 0;
}

/// [CHECK BUTTON]
pw_widget pw_create_checkbutton(	pw_widget parent,
	                              int x, int y, unsigned width, unsigned height, ...)
{
	pw_widget widget = NULL;
	va_list vl;

	widget = new_widget(CHECKBUTTON, parent, x, y, width, height, "");
	widget->redraw(widget);
	
	_pw_redraw_queue_add(widget, 1);

	widget->specific.checkbutton.set_font(widget, PW_NORMAL_FONT);
	widget->specific.checkbutton.set_justification(widget, JUSTIFY_LEFT);
	widget->specific.checkbutton.set_disabled_font_color(widget, "grey");
	
	va_start(vl, height);
	_pw_serve_options(widget, vl);
	va_end(vl);
	
	return widget;
}

void pw_checkbutton_select(pw_widget widget)
{
	widget->specific.checkbutton.select(widget);
	_pw_redraw_queue_add(widget, 1);
}

void pw_checkbutton_deselect(pw_widget widget)
{
	widget->specific.checkbutton.deselect(widget);
	_pw_redraw_queue_add(widget, 1);
}

unsigned pw_checkbutton_status(pw_widget widget)
{
	if(widget->specific.checkbutton.value)
		return 1;
	else
		return 0;
}

/// [WIDGET LISTBOX]
pw_widget pw_create_wlistbox(	pw_widget parent,
                              int x, int y, unsigned width, unsigned height, ...)
{
	pw_widget widget = NULL;
	va_list vl;

	widget = new_widget(LISTBOX, parent, x, y, width, height, "");
	widget->redraw(widget);
	
	_pw_redraw_queue_add(widget, 1);

	va_start(vl, height);
	_pw_serve_options(widget, vl);
	va_end(vl);
	
	return widget;
}

void pw_wlistbox_insert(pw_widget widget, pw_widget element, unsigned rownum)
{
	widget->specific.listbox.set_child(widget, element, rownum);
	widget->show(widget);
	widget->redraw(widget);
}

void pw_wlistbox_insertend(pw_widget widget, pw_widget element)
{
	widget->specific.listbox.set_child(widget, element, widget->specific.listbox.no_of_children);
	widget->show(widget);
	widget->redraw(widget);
}

void pw_wlistbox_remove(pw_widget widget, unsigned rownum)
{
	pw_widget child;

	child = widget->specific.listbox.get_child(widget, rownum);
	widget->specific.listbox.remove_child(widget, rownum);
	child->destruct(child);
	
	_pw_redraw_queue_add(widget, 1);
}

void pw_wlistbox_clear(pw_widget widget)
{
	int i, len;
	pw_widget child;

	if(!widget)
		return;

	widget->specific.listbox.see(widget, 0);
	widget->redraw(widget);

	len = widget->specific.listbox.no_of_children;
	for(i=0; i<len; i++) {
          child = widget->specific.listbox.get_child(widget, 0);
		widget->specific.listbox.remove_child(widget, 0);
		child->hide(child);
		child->destruct(child);
	}
		
	_pw_redraw_queue_add(widget, 1);
}

pw_widget pw_wlistbox_get_element(pw_widget widget, unsigned rownum)
{
	return widget->specific.listbox.get_child(widget, rownum); 
}

int pw_wlistbox_get_selection(pw_widget widget)
{
	return widget->specific.listbox.event_within_index;
}

void pw_wlistbox_see(pw_widget widget, unsigned rownum)
{
	widget->specific.listbox.see(widget, rownum);
}

int pw_wlistbox_get_numrows(pw_widget widget)
{
	return widget->specific.listbox.no_of_children;
}

/// [TEXT LISTBOX]
pw_widget pw_create_tlistbox(	pw_widget parent,
                              int x, int y, unsigned width, unsigned height, ...)
{
	pw_widget widget = NULL;
	va_list vl;

	widget = new_widget(TXTLB, parent, x, y, width, height, "");
	widget->redraw(widget);
	
	_pw_redraw_queue_add(widget, 1);

	va_start(vl, height);
	_pw_serve_options(widget, vl);
	va_end(vl);
	
	return widget;
}

void pw_tlistbox_addcolumn(pw_widget widget, unsigned width)
{
	add_lb_column(widget, width);
	_pw_redraw_queue_add(widget, 1);
}

void pw_tlistbox_insert(pw_widget widget, char *text, unsigned rownum, unsigned colnum)
{
	widget->specific.txtlb.insert_lb_elem(widget, text, colnum, rownum);
	_pw_redraw_queue_add(widget, 1);
}

void pw_tlistbox_insertend(pw_widget widget, char *text, unsigned colnum)
{
	add_lb_elem(widget, text, colnum);
	_pw_redraw_queue_add(widget, 1);
}

void pw_tlistbox_remove(pw_widget widget, unsigned row, Bool refresh_fg)
{
	remove_lb_row(widget, row, refresh_fg);
	_pw_redraw_queue_add(widget, 1);
}

void pw_tlistbox_clear(pw_widget widget)
{
	txtlb_clear_all_rows(widget);
	_pw_redraw_queue_add(widget, 1);
}

void pw_tlistbox_change(pw_widget widget, char *text, unsigned rownum, unsigned colnum)
{
	change_lb_elem(widget, text, colnum, rownum);
	_pw_redraw_queue_add(widget, 1);
}

int pw_tlistbox_get_selection(pw_widget widget)
{
	return txtlb_get_selecn(widget);
}

int pw_tlistbox_get_multiple_selection(pw_widget widget, int **selection_index)
{
	return txtlb_get_all_selecn(widget, selection_index);
}

void pw_tlistbox_set_selection(pw_widget widget, unsigned rownum)
{
	txtlb_set_selection(widget, rownum);
	_pw_redraw_queue_add(widget, 1);
}

void pw_tlistbox_get_element(pw_widget widget, char **text, unsigned rownum, unsigned colnum)
{
	txtlb_get_element(widget, text, rownum, colnum);
}

int pw_tlistbox_get_numrows(pw_widget widget)
{
	return txtlb_get_no_of_rows(widget);
}

void pw_tlistbox_see(pw_widget widget, unsigned rownum)
{
	txtlb_set_view(widget, rownum);
	_pw_redraw_queue_add(widget, 1);
}

void pw_tlistbox_selection_intimate(pw_widget widget, 
								void (*change_fn)(pw_widget widget, int rownum))
{
	txtlb_set_intimate_change(widget, change_fn);
}

void pw_tlistbox_set_selection_type(pw_widget widget, unsigned selection_type)
{
	switch(selection_type) {
		case PW_TLISTBOX_SINGLE_SELECTION:
			txtlb_set_type(widget, TXTLB_SINGLE_SELECN);
			break;
		case PW_TLISTBOX_MULTIPLE_SELECTION:
			txtlb_set_type(widget, TXTLB_MULTI_SELECN);
			break;
		default:
			break;
	}
	_pw_redraw_queue_add(widget, 1);
}

/// [COMBO BOX]
pw_widget pw_create_combobox( pw_widget parent,
							int x, int y, unsigned width, unsigned height, ...)
{
	pw_widget widget = NULL;
	va_list vl;

	widget = new_widget(COMBOBOX, parent, x, y, width, height, "");
	widget->redraw(widget);
	
	_pw_redraw_queue_add(widget, 1);

	va_start(vl, height);
	_pw_serve_options(widget, vl);
	va_end(vl);
	
	return widget;
}

int pw_combobox_insert(pw_widget widget, char *text, Bool refresh_fg)
{
	int ret;

	ret = widget->specific.combobox.add_option(widget, text, refresh_fg);
	if(refresh_fg)
		_pw_redraw_queue_add(widget, 1);

	return ret;
}

int pw_combobox_remove(pw_widget widget, unsigned index)
{
	int ret;

	ret = widget->specific.combobox.remove_option(widget, index);
	_pw_redraw_queue_add(widget, 1);

	return ret;
}

int pw_combobox_clear(pw_widget widget)
{
	int ret;
	
	ret = widget->specific.combobox.clear_options(widget);
	_pw_redraw_queue_add(widget, 1);

	return ret;
}

void pw_combobox_get_selection(pw_widget widget, char **text, unsigned *index)
{
	widget->specific.combobox.get_selecn(widget, text, index);
	_pw_redraw_queue_add(widget, 1);
}

void pw_combobox_set_selection(pw_widget widget, unsigned index)
{
	widget->specific.combobox.set_selecn(widget, index);
	_pw_redraw_queue_add(widget, 1);
}

void pw_combobox_selection_intimate(pw_widget widget,
								void (*change_fn)(pw_widget widget, int index))
{
	widget->specific.combobox.intimate_selecn_change(widget, change_fn);
}

/// [TAB]
pw_widget pw_create_tab(	pw_widget parent,
                                   int x, int y, unsigned width, unsigned height, ...)
{
	pw_widget widget = NULL;
	va_list vl;

	widget = new_widget(TAB, parent, x, y, width, height, "");
	widget->redraw(widget);
	
	_pw_redraw_queue_add(widget, 1);

	va_start(vl, height);
	_pw_serve_options(widget, vl);
	va_end(vl);
	
	return widget;
}

pw_widget pw_tab_add_page(pw_widget widget, const char *title, Bool(*tab_callback)(pw_widget, pw_event))
{
	pw_widget page;
	
	page = widget->specific.tab.add_tab(widget, title, tab_callback);

	return page;
}

void pw_tab_raise_page(pw_widget widget, unsigned tabnum)
{
	widget->specific.tab.raise_tab(widget, tabnum);
}

pw_widget pw_tab_get_page(pw_widget widget, unsigned tabnum)
{
	return widget->specific.tab.get_widget_pointer(widget, tabnum);
}

pw_widget pw_tab_get_active_page(pw_widget widget)
{
	return widget->specific.tab.get_active_widget(widget);
}

/// [PROGRESS BAR]
pw_widget pw_create_progressbar(	pw_widget parent,
                              	int x, int y, unsigned width, unsigned height, ...)
{
	pw_widget widget = NULL;
	va_list vl;
	px_color col;

	widget = new_widget(PROGRESSBAR, parent, x, y, width, height, "");
	widget->redraw(widget);
	
	_pw_redraw_queue_add(widget, 1);

	col.type = SOLID;
     widget_xcolor_from_str(&col.first_color, "blue");

	widget->specific.progressbar.set_fill_color(widget, &col);
	widget->specific.progressbar.set_font_color(widget, "white");
	widget->specific.progressbar.set_value(widget, 0);
	widget->specific.progressbar.show_value(widget, True);
	
	va_start(vl, height);
	_pw_serve_options(widget, vl);
	va_end(vl);
	
	return widget;
}

void pw_progressbar_setvalue(pw_widget widget, unsigned value)
{
	if(widget->wtype==PROGRESSBAR) {
		if(value>=0 && value<100) {
			widget->specific.progressbar.set_value(widget, value);
			_pw_redraw_queue_add(widget, 1);
		}
	}
}

void pw_progressbar_setcolor(pw_widget widget, const char *color)
{
	if(widget->wtype==PROGRESSBAR) {
		px_color col;

		col.type = SOLID;
		widget_xcolor_from_str(&col.first_color, color);

		widget->specific.progressbar.set_fill_color(widget, &col);

		_pw_redraw_queue_add(widget, 1);
	}
}

void pw_progressbar_showvalue(pw_widget widget)
{
	if(widget->wtype==PROGRESSBAR) {
		widget->specific.progressbar.show_value(widget, True);
		_pw_redraw_queue_add(widget, 1);
	}
}

void pw_progressbar_hidevalue(pw_widget widget)
{
	if(widget->wtype==PROGRESSBAR) {
		widget->specific.progressbar.show_value(widget, False);
		_pw_redraw_queue_add(widget, 1);
	}
}

/// [TEXT BOX]
pw_widget pw_create_textbox(	pw_widget parent,
                                   int x, int y, unsigned width, unsigned height, ...)
{
	pw_widget widget = NULL;
	va_list vl;

	widget = new_widget(TEXT, parent, x, y, width, height, "");
	widget->redraw(widget);
	
	_pw_redraw_queue_add(widget, 1);

	va_start(vl, height);
	_pw_serve_options(widget, vl);
	va_end(vl);
	
	return widget;
}

void pw_textbox_loadfile(pw_widget widget, const char *filename)
{
	widget->specific.text.load_file(widget, filename);
	_pw_redraw_queue_add(widget, 1);
}

void pw_textbox_savefile(pw_widget widget, const char *filename)
{
	widget->specific.text.save_file(widget, filename);
	_pw_redraw_queue_add(widget, 1);
}

int pw_textbox_get_numlines(pw_widget widget)
{
	return widget->specific.text.get_numlines(widget);
}

const char *pw_textbox_get_line(pw_widget widget, unsigned linenum)
{
	return widget->specific.text.get_line(widget, linenum);
}

/// [SKETCH]
pw_widget pw_create_sketch(	pw_widget parent,
                                   int x, int y, unsigned width, unsigned height, ...)
{
	pw_widget widget = NULL;
	va_list vl;

	widget = new_widget(SKETCH, parent, x, y, width, height, "");
	widget->redraw(widget);
	widget->show(widget);
	
	_pw_redraw_queue_add(widget, 1);

	va_start(vl, height);
	_pw_serve_options(widget, vl);
	va_end(vl);
	
	return widget;
}

void pw_sketch_loadfile(pw_widget widget, const char *filename)
{
	widget->specific.sketch.load_from_file(widget, filename);
	_pw_redraw_queue_add(widget, 1);
}

void pw_sketch_savefile(pw_widget widget, const char *filename)
{
	widget->specific.sketch.save_to_file(widget, filename);
}

void pw_sketch_pencolor(pw_widget widget, const char *color)
{
	px_color col;
	unsigned long scol = 0;
	char str[20];

	col.type = SOLID;
	widget_xcolor_from_str(&col.first_color, color);

	sprintf(str, "0x%02x%02x%02x", 
			col.first_color.red*255/0xffff, 
			col.first_color.green*255/0xffff, 
			col.first_color.blue*255/0xffff);
	sscanf(str, "%lx", &scol);

	widget->specific.sketch.set_pen_color(widget, scol);
	_pw_redraw_queue_add(widget, 1);
}

void pw_sketch_penwidth(pw_widget widget, unsigned width)
{
	widget->specific.sketch.set_pen_width(widget, width);
	_pw_redraw_queue_add(widget, 1);
}

void pw_sketch_clear(pw_widget widget)
{
	widget->specific.sketch.clear_sketches(widget);
	_pw_redraw_queue_add(widget, 1);
}

/// [VERTICAL SCROLLBAR]
pw_widget pw_create_vscrollbar(	pw_widget parent,
                                   int x, int y, unsigned width, unsigned height, ...)
{
	pw_widget widget = NULL;
	va_list vl;

	widget = new_widget(VSCROLLBAR, parent, x, y, width, height, "");
	widget->redraw(widget);
	
	_pw_redraw_queue_add(widget, 1);

	va_start(vl, height);
	_pw_serve_options(widget, vl);
	va_end(vl);
	
	return widget;
}

void pw_vscrollbar_set_sliderpos(pw_widget widget, float position)
{
	widget->specific.vscrollbar.set_slider_position(widget, position);
	_pw_redraw_queue_add(widget, 1);
}

void pw_vscrollbar_set_sliderlen(pw_widget widget, float length)
{
	widget->specific.vscrollbar.set_slider_position(widget, length);
	_pw_redraw_queue_add(widget, 1);
}

void pw_vscrollbar_set_sliderincr(pw_widget widget, float increment)
{
	widget->specific.vscrollbar.set_increment(widget, increment);
}

void pw_vscrollbar_intimate_change(pw_widget widget, void (*callback)(px_widget *widget, float position))
{
	widget->specific.vscrollbar.intimate_change = callback;
}

float pw_vscrollbar_get_sliderpos(pw_widget widget)
{
	return widget->specific.vscrollbar.get_slider_position(widget);
}

float pw_vscrollbar_get_sliderincr(pw_widget widget)
{
	return widget->specific.vscrollbar.get_increment(widget);
}

/// [HORIZONTAL SCROLLBAR]
pw_widget pw_create_hscrollbar(	pw_widget parent,
                                   int x, int y, unsigned width, unsigned height, ...)
{
	pw_widget widget = NULL;
	va_list vl;

	widget = new_widget(HSCROLLBAR, parent, x, y, width, height, "");
	widget->redraw(widget);
	
	_pw_redraw_queue_add(widget, 1);

	va_start(vl, height);
	_pw_serve_options(widget, vl);
	va_end(vl);
	
	return widget;
}

void pw_hscrollbar_set_sliderpos(pw_widget widget, float position)
{
	widget->specific.hscrollbar.set_slider_position(widget, position);
	_pw_redraw_queue_add(widget, 1);
}

void pw_hscrollbar_set_sliderlen(pw_widget widget, float length)
{
	widget->specific.hscrollbar.set_slider_position(widget, length);
	_pw_redraw_queue_add(widget, 1);
}

void pw_hscrollbar_set_sliderincr(pw_widget widget, float increment)
{
	widget->specific.hscrollbar.set_increment(widget, increment);
}

void pw_hscrollbar_intimate_change(pw_widget widget, void (*callback)(px_widget *widget, float position))
{
	widget->specific.hscrollbar.intimate_change = callback;
}

float pw_hscrollbar_get_sliderpos(pw_widget widget)
{
	return widget->specific.hscrollbar.get_slider_position(widget);
}

float pw_hscrollbar_get_sliderincr(pw_widget widget)
{
	return widget->specific.hscrollbar.get_increment(widget);
}

/// [VERTICAL SLIDER]
pw_widget pw_create_vslider(	pw_widget parent,
                                   int x, int y, unsigned width, unsigned height, ...)
{
	pw_widget widget = NULL;
	va_list vl;

	widget = new_widget(VSLIDER, parent, x, y, width, height, "");
	widget->redraw(widget);
	
	_pw_redraw_queue_add(widget, 1);

	va_start(vl, height);
	_pw_serve_options(widget, vl);
	va_end(vl);
	
	return widget;
}

void pw_vslider_set_range(pw_widget widget, int min, int max)
{
	widget->specific.vslider.set_range(widget, min, max);
	_pw_redraw_queue_add(widget, 1);
}

void pw_vslider_set_position(pw_widget widget, int position)
{
	widget->specific.vslider.set_position(widget, position); 
	_pw_redraw_queue_add(widget, 1);
}

int pw_vslider_get_position(pw_widget widget, int position)
{
	return widget->specific.vslider.get_position(widget); 
}

void pw_vslider_show_value(pw_widget widget, unsigned calibration_side)
{
	if(calibration_side==PW_SLIDER_LEFT) 
		widget->specific.vslider.show_calibration(widget, SLIDER_LEFT);

	if(calibration_side==PW_SLIDER_RIGHT)
		widget->specific.vslider.show_calibration(widget, SLIDER_RIGHT);

	widget->specific.vslider.show_value(widget, True);
	_pw_redraw_queue_add(widget, 1);
}

void pw_vslider_hide_value(pw_widget widget)
{
	widget->specific.vslider.show_calibration(widget, VSLIDER_NONE);
	widget->specific.vslider.show_value(widget, False);
	_pw_redraw_queue_add(widget, 1);
}

void pw_vslider_intimate_release(pw_widget widget, void (*callback)(int position))
{
	widget->specific.vslider.intimate_slider_release = callback;
}

void pw_vslider_intimate_motion(pw_widget widget, void (*callback)(int position))
{
	widget->specific.vslider.intimate_slider_motion = callback;
}

/// [HORIZONTAL SLIDER]
pw_widget pw_create_hslider(	pw_widget parent,
                                   int x, int y, unsigned width, unsigned height, ...)
{
	pw_widget widget = NULL;
	va_list vl;

	widget = new_widget(HSLIDER, parent, x, y, width, height, "");
	widget->redraw(widget);
	
	_pw_redraw_queue_add(widget, 1);

	va_start(vl, height);
	_pw_serve_options(widget, vl);
	va_end(vl);
	
	return widget;
}

void pw_hslider_set_range(pw_widget widget, int min, int max)
{
	widget->specific.hslider.set_range(widget, min, max);
	_pw_redraw_queue_add(widget, 1);
}

void pw_hslider_set_position(pw_widget widget, int position)
{
	widget->specific.hslider.set_position(widget, position); 
	_pw_redraw_queue_add(widget, 1);
}

int pw_hslider_get_position(pw_widget widget, int position)
{
	return widget->specific.hslider.get_position(widget); 
}

void pw_hslider_show_value(pw_widget widget, unsigned calibration_side)
{
	if(calibration_side==PW_SLIDER_TOP) 
		widget->specific.hslider.show_calibration(widget, SLIDER_TOP);

	if(calibration_side==PW_SLIDER_BOTTOM)
		widget->specific.hslider.show_calibration(widget, SLIDER_BOTTOM);

	widget->specific.hslider.show_value(widget, True);
	_pw_redraw_queue_add(widget, 1);
}

void pw_hslider_hide_value(pw_widget widget)
{
	widget->specific.hslider.show_calibration(widget, HSLIDER_NONE);
	widget->specific.hslider.show_value(widget, False);
	_pw_redraw_queue_add(widget, 1);
}

void pw_hslider_intimate_release(pw_widget widget, void (*callback)(int position))
{
	widget->specific.hslider.intimate_slider_release = callback;
}

void pw_hslider_intimate_motion(pw_widget widget, void (*callback)(int position))
{
	widget->specific.hslider.intimate_slider_motion = callback;
}
